﻿using System;
using System.IO.Ports;
using System.Windows.Forms;

namespace Mackoy.Bvets.TsmcInterface
{
    public class TsmcInterface : Mackoy.Bvets.IInputDevice
    {
        private SerialPort myPort = null;
        private string lastWord = string.Empty;

        public static Settings settings = null;

        public event InputEventHandler KeyDown;
        public event InputEventHandler KeyUp;
        public event InputEventHandler LeverMoved;

        public void Load(string settingsPath)
        {
            settings = Settings.LoadFromXml(settingsPath);

            openPort();

            if (myPort != null)
            {
                try
                {
                    myPort.Write(new byte[] { 0 }, 0, 1);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "TS Master Controller Interface", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }

        public void SetAxisRanges(int[][] ranges)
        {
        }

        public void Tick()
        {
            if (myPort == null)
            {
                return;
            }

            string text;
            try
            {
                text = myPort.ReadExisting();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "TS Master Controller Interface", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                closePort();
                return;
            }
            if (text.Length == 0)
            {
                return;
            }

            string[] words = (lastWord + text).Split('\r');
            for (int i = 0; i < words.Length - 1; i++)
            {
                switch (words[i])
                {
                    case "TSB20":
                        onLeverMoved(3, -9);
                        break;
                    case "TSB30":
                        onLeverMoved(3, -8);
                        break;
                    case "TSB40":
                        onLeverMoved(3, -7);
                        break;
                    case "TSE99":
                        onLeverMoved(3, -6);
                        break;
                    case "TSA05":
                        onLeverMoved(3, -5);
                        break;
                    case "TSA15":
                        onLeverMoved(3, -4);
                        break;
                    case "TSA25":
                        onLeverMoved(3, -3);
                        break;
                    case "TSA35":
                        onLeverMoved(3, -2);
                        break;
                    case "TSA45":
                        onLeverMoved(3, -1);
                        break;
                    case "TSA50":
                        onLeverMoved(3, 0);
                        break;
                    case "TSA55":
                        onLeverMoved(3, 1);
                        break;
                    case "TSA65":
                        onLeverMoved(3, 2);
                        break;
                    case "TSA75":
                        onLeverMoved(3, 3);
                        break;
                    case "TSA85":
                        onLeverMoved(3, 4);
                        break;
                    case "TSA95":
                        onLeverMoved(3, 5);
                        break;
                    case "TSB60":
                        onLeverMoved(3, 6);
                        break;
                    case "TSB70":
                        onLeverMoved(3, 7);
                        break;
                    case "TSB80":
                        onLeverMoved(3, 8);
                        break;
                    case "TSG99":
                        onLeverMoved(0, 1);
                        break;
                    case "TSG50":
                        onLeverMoved(0, 0);
                        break;
                    case "TSG00":
                        onLeverMoved(0, -1);
                        break;
                    case "TSK99":
                        onKeyDown(settings.ButtonS[0], settings.ButtonS[1]);
                        break;
                    case "TSK00":
                        onKeyUp(settings.ButtonS[0], settings.ButtonS[1]);
                        break;
                    case "TSX99":
                        onKeyDown(settings.ButtonA[0], settings.ButtonA[1]);
                        break;
                    case "TSX00":
                        onKeyUp(settings.ButtonA[0], settings.ButtonA[1]);
                        break;
                    case "TSY99":
                        onKeyDown(settings.ButtonB[0], settings.ButtonB[1]);
                        break;
                    case "TSY00":
                        onKeyUp(settings.ButtonB[0], settings.ButtonB[1]);
                        break;
                    case "TSZ99":
                        onKeyDown(settings.ButtonC[0], settings.ButtonC[1]);
                        break;
                    case "TSZ00":
                        onKeyUp(settings.ButtonC[0], settings.ButtonC[1]);
                        break;
                }
            }

            lastWord = words[words.Length - 1];
        }

        public void Configure(IWin32Window owner)
        {
            using (ConfigForm form = new ConfigForm())
            {
                form.ShowDialog(owner);
            }
        }

        public void Dispose()
        {
            closePort();

            if (settings != null)
            {
                settings.SaveToXml();
                settings = null;
            }
        }

        private void openPort()
        {
            try
            {
                myPort = new SerialPort("COM" + settings.Port.ToString(), 19200, Parity.None, 8, StopBits.One);
                myPort.Open();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "TS Master Controller Interface", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                closePort();
            }
        }

        private void closePort()
        {
            if (myPort != null)
            {
                try
                {
                    myPort.Close();
                }
                catch { }
                try
                {
                    myPort.Dispose();
                }
                catch { }
                myPort = null;
            }
        }

        private void onLeverMoved(int axis, int notch)
        {
            if (LeverMoved != null)
            {
                LeverMoved(this, new InputEventArgs(axis, notch));
            }
        }

        private void onKeyDown(int axis, int keyCode)
        {
            if (LeverMoved != null)
            {
                KeyDown(this, new InputEventArgs(axis, keyCode));
            }
        }

        private void onKeyUp(int axis, int keyCode)
        {
            if (LeverMoved != null)
            {
                KeyUp(this, new InputEventArgs(axis, keyCode));
            }
        }
    }
}
